Object.defineProperty(exports, "__esModule", { value: true });
var nativescript_observable_1 = require("nativescript-observable");
var view_1 = require("@nativescript/core/ui/core/view");
var platform_1 = require("@nativescript/core/platform");
exports.GestureHandlerStateEvent = 'GestureHandlerStateEvent';
exports.GestureHandlerTouchEvent = 'GestureHandlerTouchEvent';
var HandlerType;
(function (HandlerType) {
    HandlerType["PAN"] = "pan";
    HandlerType["TAP"] = "tap";
    HandlerType["FLING"] = "fling";
    HandlerType["LONG_PRESS"] = "longPress";
    HandlerType["NATIVE_VIEW"] = "nativeView";
    HandlerType["PINCH"] = "pinch";
    HandlerType["ROTATION"] = "rotation";
    HandlerType["FORCE_TOUCH"] = "forceTouch";
})(HandlerType = exports.HandlerType || (exports.HandlerType = {}));
function createGetter(key, options) {
    var nativeGetterName = ((platform_1.isAndroid ? options.android : options.ios) || options).nativeGetterName || 'get' + key.charAt(0).toUpperCase() + key.slice(1);
    var converter = options.converter;
    return function () {
        var result;
        if (this.native && this.native[nativeGetterName]) {
            result = this.native[nativeGetterName]();
        }
        else {
            result = this.options[key] || options.defaultValue;
        }
        result = converter && converter.fromNative ? converter.fromNative.call(this, result, key) : result;
        return result;
    };
}
function createSetter(key, options) {
    var nativeSetterName = ((platform_1.isAndroid ? options.android : options.ios) || options).nativeSetterName || 'set' + key.charAt(0).toUpperCase() + key.slice(1);
    return function (newVal) {
        this.options[key] = newVal;
        if (this.native && this.native[nativeSetterName]) {
            var actualVal = options.converter && options.converter.toNative ? options.converter.toNative.call(this, newVal, key) : newVal;
            this.native[nativeSetterName].call(this.native, actualVal);
        }
    };
}
function nativePropertyGenerator(target, key, options) {
    Object.defineProperty(target, key, {
        get: createGetter(key, options),
        set: createSetter(key, options),
        enumerable: true,
        configurable: true
    });
}
function nativeProperty() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    if (args.length === 1) {
        return function (target, key, descriptor) {
            return nativePropertyGenerator(target, key, args[0] || {});
        };
    }
    else {
        var options = typeof args[1] === 'string' ? undefined : args[0];
        var startIndex = !!options ? 1 : 0;
        return nativePropertyGenerator(args[startIndex], args[startIndex + 1], options || {});
    }
}
exports.nativeProperty = nativeProperty;
var BaseNative = (function (_super) {
    __extends(BaseNative, _super);
    function BaseNative(options, native) {
        var _this = _super.call(this) || this;
        _this.options = options;
        _this.getNative = function () {
            if (!_this.native) {
                _this.native = _this.createNative(_this.options);
                _this.initNativeView(_this.native, _this.options);
            }
            return _this.native;
        };
        if (native) {
            _this.native = native;
        }
        return _this;
    }
    BaseNative.prototype.initNativeView = function (native, options) {
        for (var key in options) {
            this[key] = options[key];
        }
    };
    BaseNative.prototype.disposeNativeView = function () {
        this.native = null;
    };
    BaseNative.prototype.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        console.log.apply(console, __spreadArrays(["[" + this.constructor.name + "]"], args));
    };
    return BaseNative;
}(nativescript_observable_1.default));
exports.BaseNative = BaseNative;
var Manager = (function (_super) {
    __extends(Manager, _super);
    function Manager() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return Manager;
}(nativescript_observable_1.default));
exports.Manager = Manager;
var GestureState;
(function (GestureState) {
    GestureState[GestureState["UNDETERMINED"] = 0] = "UNDETERMINED";
    GestureState[GestureState["FAILED"] = 1] = "FAILED";
    GestureState[GestureState["BEGAN"] = 2] = "BEGAN";
    GestureState[GestureState["CANCELLED"] = 3] = "CANCELLED";
    GestureState[GestureState["ACTIVE"] = 4] = "ACTIVE";
    GestureState[GestureState["END"] = 5] = "END";
})(GestureState = exports.GestureState || (exports.GestureState = {}));
function applyMixins(derivedCtor, baseCtors, options) {
    var omits = options && options.omit ? options.omit : [];
    baseCtors.forEach(function (baseCtor) {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach(function (name) {
            if (omits.indexOf(name) !== -1) {
                return;
            }
            var descriptor = Object.getOwnPropertyDescriptor(baseCtor.prototype, name);
            if (name === 'constructor')
                return;
            if (descriptor && (!descriptor.writable || !descriptor.configurable || !descriptor.enumerable || descriptor.get || descriptor.set)) {
                Object.defineProperty(derivedCtor.prototype, name, descriptor);
            }
            else {
                var oldImpl_1 = derivedCtor.prototype[name];
                if (!oldImpl_1) {
                    derivedCtor.prototype[name] = baseCtor.prototype[name];
                }
                else {
                    derivedCtor.prototype[name] = function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i] = arguments[_i];
                        }
                        if (options) {
                            if (!!options.override) {
                                return baseCtor.prototype[name].apply(this, args);
                            }
                            else if (!!options.after) {
                                oldImpl_1.apply(this, args);
                                return baseCtor.prototype[name].apply(this, args);
                            }
                            else {
                                baseCtor.prototype[name].apply(this, args);
                                return oldImpl_1.apply(this, args);
                            }
                        }
                        else {
                            baseCtor.prototype[name].apply(this, args);
                            return oldImpl_1.apply(this, args);
                        }
                    };
                }
            }
        });
        Object.getOwnPropertySymbols(baseCtor.prototype).forEach(function (symbol) {
            if (omits.indexOf(symbol) !== -1) {
                return;
            }
            var oldImpl = derivedCtor.prototype[symbol];
            if (!oldImpl) {
                derivedCtor.prototype[symbol] = baseCtor.prototype[symbol];
            }
            else {
                derivedCtor.prototype[symbol] = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    if (options) {
                        if (!!options.override) {
                            return baseCtor.prototype[symbol].apply(this, args);
                        }
                        else if (!!options.after) {
                            oldImpl.apply(this, args);
                            return baseCtor.prototype[symbol].apply(this, args);
                        }
                        else {
                            baseCtor.prototype[symbol].apply(this, args);
                            return oldImpl.apply(this, args);
                        }
                    }
                    else {
                        baseCtor.prototype[symbol].apply(this, args);
                        return oldImpl.apply(this, args);
                    }
                };
            }
        });
    });
}
exports.applyMixins = applyMixins;
exports.ViewInitEvent = 'ViewInitEvent';
exports.ViewDisposeEvent = 'ViewDisposeEvent';
var ViewGestureExtended = (function (_super) {
    __extends(ViewGestureExtended, _super);
    function ViewGestureExtended() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ViewGestureExtended.prototype.initNativeView = function () {
        if (this.nativeView) {
            this.nativeView.nsView = new WeakRef(this);
        }
        this.notify({ eventName: exports.ViewInitEvent, object: this });
    };
    ViewGestureExtended.prototype.disposeNativeView = function () {
        if (this.nativeView) {
            this.nativeView.nsView = null;
        }
        this.notify({ eventName: exports.ViewDisposeEvent, object: this });
    };
    return ViewGestureExtended;
}(view_1.View));
var installed = false;
function overrideViewBase() {
    var NSView = require('@nativescript/core/ui/core/view').View;
    applyMixins(NSView, [ViewGestureExtended]);
}
exports.overrideViewBase = overrideViewBase;
function install() {
    if (installed) {
        return;
    }
    installed = true;
    overrideViewBase();
}
exports.install = install;
//# sourceMappingURL=gesturehandler.common.js.map