Object.defineProperty(exports, "__esModule", { value: true });
const nativescript_observable_1 = require("nativescript-observable");
const view_1 = require("@nativescript/core/ui/core/view");
const platform_1 = require("@nativescript/core/platform");
exports.GestureHandlerStateEvent = 'GestureHandlerStateEvent';
exports.GestureHandlerTouchEvent = 'GestureHandlerTouchEvent';
var HandlerType;
(function (HandlerType) {
    HandlerType["PAN"] = "pan";
    HandlerType["TAP"] = "tap";
    HandlerType["FLING"] = "fling";
    HandlerType["LONG_PRESS"] = "longPress";
    HandlerType["NATIVE_VIEW"] = "nativeView";
    HandlerType["PINCH"] = "pinch";
    HandlerType["ROTATION"] = "rotation";
    HandlerType["FORCE_TOUCH"] = "forceTouch";
})(HandlerType = exports.HandlerType || (exports.HandlerType = {}));
function createGetter(key, options) {
    const nativeGetterName = ((platform_1.isAndroid ? options.android : options.ios) || options).nativeGetterName || 'get' + key.charAt(0).toUpperCase() + key.slice(1);
    const converter = options.converter;
    return function () {
        let result;
        if (this.native && this.native[nativeGetterName]) {
            result = this.native[nativeGetterName]();
        }
        else {
            result = this.options[key] || options.defaultValue;
        }
        result = converter && converter.fromNative ? converter.fromNative.call(this, result, key) : result;
        return result;
    };
}
function createSetter(key, options) {
    const nativeSetterName = ((platform_1.isAndroid ? options.android : options.ios) || options).nativeSetterName || 'set' + key.charAt(0).toUpperCase() + key.slice(1);
    return function (newVal) {
        this.options[key] = newVal;
        if (this.native && this.native[nativeSetterName]) {
            const actualVal = options.converter && options.converter.toNative ? options.converter.toNative.call(this, newVal, key) : newVal;
            this.native[nativeSetterName].call(this.native, actualVal);
        }
    };
}
function nativePropertyGenerator(target, key, options) {
    Object.defineProperty(target, key, {
        get: createGetter(key, options),
        set: createSetter(key, options),
        enumerable: true,
        configurable: true
    });
}
function nativeProperty(...args) {
    if (args.length === 1) {
        return function (target, key, descriptor) {
            return nativePropertyGenerator(target, key, args[0] || {});
        };
    }
    else {
        const options = typeof args[1] === 'string' ? undefined : args[0];
        const startIndex = !!options ? 1 : 0;
        return nativePropertyGenerator(args[startIndex], args[startIndex + 1], options || {});
    }
}
exports.nativeProperty = nativeProperty;
class BaseNative extends nativescript_observable_1.default {
    constructor(options, native) {
        super();
        this.options = options;
        this.getNative = () => {
            if (!this.native) {
                this.native = this.createNative(this.options);
                this.initNativeView(this.native, this.options);
            }
            return this.native;
        };
        if (native) {
            this.native = native;
        }
    }
    initNativeView(native, options) {
        for (const key in options) {
            this[key] = options[key];
        }
    }
    disposeNativeView() {
        this.native = null;
    }
    log(...args) {
        console.log(`[${this.constructor.name}]`, ...args);
    }
}
exports.BaseNative = BaseNative;
class Manager extends nativescript_observable_1.default {
}
exports.Manager = Manager;
var GestureState;
(function (GestureState) {
    GestureState[GestureState["UNDETERMINED"] = 0] = "UNDETERMINED";
    GestureState[GestureState["FAILED"] = 1] = "FAILED";
    GestureState[GestureState["BEGAN"] = 2] = "BEGAN";
    GestureState[GestureState["CANCELLED"] = 3] = "CANCELLED";
    GestureState[GestureState["ACTIVE"] = 4] = "ACTIVE";
    GestureState[GestureState["END"] = 5] = "END";
})(GestureState = exports.GestureState || (exports.GestureState = {}));
function applyMixins(derivedCtor, baseCtors, options) {
    const omits = options && options.omit ? options.omit : [];
    baseCtors.forEach(baseCtor => {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach(name => {
            if (omits.indexOf(name) !== -1) {
                return;
            }
            const descriptor = Object.getOwnPropertyDescriptor(baseCtor.prototype, name);
            if (name === 'constructor')
                return;
            if (descriptor && (!descriptor.writable || !descriptor.configurable || !descriptor.enumerable || descriptor.get || descriptor.set)) {
                Object.defineProperty(derivedCtor.prototype, name, descriptor);
            }
            else {
                const oldImpl = derivedCtor.prototype[name];
                if (!oldImpl) {
                    derivedCtor.prototype[name] = baseCtor.prototype[name];
                }
                else {
                    derivedCtor.prototype[name] = function (...args) {
                        if (options) {
                            if (!!options.override) {
                                return baseCtor.prototype[name].apply(this, args);
                            }
                            else if (!!options.after) {
                                oldImpl.apply(this, args);
                                return baseCtor.prototype[name].apply(this, args);
                            }
                            else {
                                baseCtor.prototype[name].apply(this, args);
                                return oldImpl.apply(this, args);
                            }
                        }
                        else {
                            baseCtor.prototype[name].apply(this, args);
                            return oldImpl.apply(this, args);
                        }
                    };
                }
            }
        });
        Object.getOwnPropertySymbols(baseCtor.prototype).forEach(symbol => {
            if (omits.indexOf(symbol) !== -1) {
                return;
            }
            const oldImpl = derivedCtor.prototype[symbol];
            if (!oldImpl) {
                derivedCtor.prototype[symbol] = baseCtor.prototype[symbol];
            }
            else {
                derivedCtor.prototype[symbol] = function (...args) {
                    if (options) {
                        if (!!options.override) {
                            return baseCtor.prototype[symbol].apply(this, args);
                        }
                        else if (!!options.after) {
                            oldImpl.apply(this, args);
                            return baseCtor.prototype[symbol].apply(this, args);
                        }
                        else {
                            baseCtor.prototype[symbol].apply(this, args);
                            return oldImpl.apply(this, args);
                        }
                    }
                    else {
                        baseCtor.prototype[symbol].apply(this, args);
                        return oldImpl.apply(this, args);
                    }
                };
            }
        });
    });
}
exports.applyMixins = applyMixins;
exports.ViewInitEvent = 'ViewInitEvent';
exports.ViewDisposeEvent = 'ViewDisposeEvent';
class ViewGestureExtended extends view_1.View {
    initNativeView() {
        if (this.nativeView) {
            this.nativeView.nsView = new WeakRef(this);
        }
        this.notify({ eventName: exports.ViewInitEvent, object: this });
    }
    disposeNativeView() {
        if (this.nativeView) {
            this.nativeView.nsView = null;
        }
        this.notify({ eventName: exports.ViewDisposeEvent, object: this });
    }
}
let installed = false;
function overrideViewBase() {
    const NSView = require('@nativescript/core/ui/core/view').View;
    applyMixins(NSView, [ViewGestureExtended]);
}
exports.overrideViewBase = overrideViewBase;
function install() {
    if (installed) {
        return;
    }
    installed = true;
    overrideViewBase();
}
exports.install = install;
//# sourceMappingURL=gesturehandler.common.mjs.map