Object.defineProperty(exports, "__esModule", { value: true });
const gesturehandler_common_1 = require("./gesturehandler.common");
exports.GestureHandlerStateEvent = gesturehandler_common_1.GestureHandlerStateEvent;
exports.GestureHandlerTouchEvent = gesturehandler_common_1.GestureHandlerTouchEvent;
exports.GestureState = gesturehandler_common_1.GestureState;
exports.HandlerType = gesturehandler_common_1.HandlerType;
exports.ViewDisposeEvent = gesturehandler_common_1.ViewDisposeEvent;
exports.ViewInitEvent = gesturehandler_common_1.ViewInitEvent;
const types_1 = require("@nativescript/core/utils/types");
const gestures_override_1 = require("./gestures_override");
const gestures_1 = require("@nativescript/core/ui/gestures");
let installed = false;
function install(overrideNGestures = false) {
    if (installed) {
        return;
    }
    installed = true;
    gesturehandler_common_1.install();
    if (overrideNGestures === true) {
        const NSView = require('@nativescript/core/ui/core/view').View;
        NSView.prototype._observe = function (type, callback, thisArg) {
            if (!this._gestureObservers[type]) {
                this._gestureObservers[type] = [];
            }
            this._gestureObservers[type].push(gestures_override_1.observe(this, type, callback, thisArg));
            if (type & gestures_1.GestureTypes.touch && this.isLoaded && !this.touchListenerIsSet) {
                this.setOnTouchListener();
            }
        };
    }
}
exports.install = install;
function toJsObject(objCObj) {
    if (objCObj === null || typeof objCObj !== 'object') {
        return objCObj;
    }
    let node, key, i, l;
    const oKeyArr = objCObj.allKeys;
    if (oKeyArr === undefined && objCObj.count !== undefined) {
        node = [];
        for (i = 0, l = objCObj.count; i < l; i++) {
            key = objCObj.objectAtIndex(i);
            node.push(toJsObject(key));
        }
    }
    else if (oKeyArr !== undefined) {
        node = {};
        for (i = 0, l = oKeyArr.count; i < l; i++) {
            key = oKeyArr.objectAtIndex(i);
            const val = objCObj.valueForKey(key);
            if (val === null) {
                node[key] = null;
                continue;
            }
            node[key] = getValueForClass(val);
        }
    }
    else {
        node = getValueForClass(objCObj);
    }
    return node;
}
function getValueForClass(val) {
    switch (types_1.getClass(val)) {
        case 'NSArray':
        case 'NSMutableArray':
            return toJsObject(val);
        case 'NSDictionary':
        case 'NSMutableDictionary':
            return toJsObject(val);
        case 'String':
            return String(val);
        case 'Boolean':
            return val;
        case 'Number':
        case 'NSDecimalNumber':
            return Number(String(val));
        case 'Date':
            return new Date(val);
        default:
            console.log("Please report this at https://github.com/farfromrefug/nativescript-gesturehandler-febase/issues: iOS toJsObject is missing a converter for class '" +
                types_1.getClass(val) +
                "'. Casting to String as a fallback.");
            return String(val);
    }
}
class HandlerDelegate extends NSObject {
    static new() {
        return super.new();
    }
    static initWithOwner(owner) {
        const impl = HandlerDelegate.new();
        impl._owner = owner;
        return impl;
    }
    gestureHandlerDidChangeStatePrevStateExtraDataView(handler, state, prevState, extraData, view) {
        const owner = this._owner && this._owner.get();
        if (owner) {
            owner.notify({
                eventName: gesturehandler_common_1.GestureHandlerStateEvent,
                object: owner,
                data: {
                    state,
                    prevState,
                    ios: view,
                    extraData: toJsObject(extraData),
                    view: view.nsView ? view.nsView.get() : null,
                },
            });
        }
    }
    gestureHandlerTouchEventOnViewStateExtraData(handler, view, state, extraData) {
        const owner = this._owner && this._owner.get();
        if (owner) {
            owner.notify({
                eventName: gesturehandler_common_1.GestureHandlerTouchEvent,
                object: owner,
                data: {
                    state,
                    ios: view,
                    extraData: toJsObject(extraData),
                    view: view.nsView ? view.nsView.get() : null,
                },
            });
        }
    }
}
exports.HandlerDelegate = HandlerDelegate;
HandlerDelegate.ObjCProtocols = [GestureHandlerDelegate];
class Handler extends gesturehandler_common_1.BaseNative {
    createNative() {
        return null;
    }
    attachToView(view) {
        const tag = this.native.tag;
        this.delegate = HandlerDelegate.initWithOwner(new WeakRef(this));
        this.native.delegate = this.delegate;
        this.manager.get().attachGestureHandler(tag, view);
    }
    detachFromView(view) {
        const tag = this.native.tag;
        this.delegate = this.native.delegate = null;
        this.manager.get().detachGestureHandler(tag, view);
    }
    getTag() {
        return this.native.tag;
    }
    setTag(value) {
    }
    getView() {
        return null;
    }
    cancel() {
        this.native.reset();
    }
}
__decorate([
    gesturehandler_common_1.nativeProperty
], Handler.prototype, "enabled", void 0);
__decorate([
    gesturehandler_common_1.nativeProperty
], Handler.prototype, "shouldCancelWhenOutside", void 0);
exports.Handler = Handler;
class Manager extends gesturehandler_common_1.Manager {
    constructor() {
        super(...arguments);
        this.viewListeners = new Map();
    }
    get manager() {
        if (!this._manager) {
            this._manager = GestureHandlerManager.alloc().init();
        }
        return this._manager;
    }
    static getInstance() {
        if (!Manager.sManager) {
            Manager.sManager = new Manager();
        }
        return Manager.sManager;
    }
    createGestureHandler(handlerName, handlerTag, config) {
        const nHandler = this.manager.createGestureHandlerTagConfig(handlerName, handlerTag, config ? NSDictionary.dictionaryWithDictionary(config) : null);
        const result = new Handler(config, nHandler);
        result.manager = new WeakRef(this);
        return result;
    }
    attachGestureHandler(handlerTag, view) {
        if (view.nativeView) {
            this.manager.attachGestureHandlerToView(handlerTag, view.nativeView);
        }
        const onInit = () => {
            this.manager.attachGestureHandlerToView(handlerTag, view.nativeView);
        };
        const onDispose = () => this.manager.dropGestureHandler(handlerTag);
        view.on(gesturehandler_common_1.ViewInitEvent, onInit, this);
        view.on(gesturehandler_common_1.ViewDisposeEvent, onDispose, this);
        let viewListeners = this.viewListeners.get(view);
        if (!viewListeners) {
            viewListeners = new Map();
            this.viewListeners.set(view, viewListeners);
        }
        viewListeners.set(handlerTag, {
            init: onInit,
            dispose: onDispose,
        });
    }
    detachGestureHandler(handlerTag, view) {
        if (view.nativeView) {
            this.manager.dropGestureHandler(handlerTag);
        }
        if (view) {
            const viewListeners = this.viewListeners.get(view);
            if (viewListeners) {
                const listeners = viewListeners.get(handlerTag);
                if (listeners) {
                    view.off(gesturehandler_common_1.ViewInitEvent, listeners.init, this);
                    view.off(gesturehandler_common_1.ViewDisposeEvent, () => listeners.dispose, this);
                    viewListeners.delete(handlerTag);
                    if (viewListeners.size === 0) {
                        this.viewListeners.delete(view);
                    }
                }
            }
        }
    }
}
exports.Manager = Manager;
//# sourceMappingURL=gesturehandler.ios.mjs.map