Object.defineProperty(exports, "__esModule", { value: true });
const gestures_1 = require("@nativescript/core/ui/gestures");
const utils_1 = require("@nativescript/core/utils/utils");
const gesturehandler_1 = require("./gesturehandler");
const gesturehandler_common_1 = require("./gesturehandler.common");
function observe(target, type, callback, context) {
    const observer = new GesturesObserver(target, callback, context);
    observer.observe(type);
    return observer;
}
exports.observe = observe;
let TAG = 0;
class GesturesObserver {
    constructor(target, callback, context) {
        this._target = target;
        this._callback = callback;
        this._context = context;
    }
    get callback() {
        return this._callback;
    }
    get target() {
        return this._target;
    }
    get context() {
        return this._context;
    }
    disconnect() {
        this._detach();
        if (this.target) {
            this.target.off('loaded', this._onTargetLoaded);
            this.target.off('unloaded', this._onTargetUnloaded);
            this._onTargetLoaded = null;
            this._onTargetUnloaded = null;
        }
        if (this.target) {
            let list = this.target.getGestureObservers(this.type);
            if (list && list.length > 0) {
                for (let i = 0; i < list.length; i++) {
                    if (list[i].callback === this.callback) {
                        break;
                    }
                }
                list.length = 0;
                this.target._gestureObservers[this.type] = undefined;
                delete this.target._gestureObservers[this.type];
            }
        }
        this._target = null;
        this._callback = null;
        this._context = null;
    }
    observe(type) {
        if (this.target) {
            this.type = type;
            this._onTargetLoaded = (args) => {
                this._attach(this.target, type);
            };
            this._onTargetUnloaded = (args) => {
                this._detach();
            };
            this.target.on('loaded', this._onTargetLoaded);
            this.target.on('unloaded', this._onTargetUnloaded);
            if (this.target.isLoaded) {
                this._attach(this.target, type);
            }
        }
    }
    _detach() {
        this._notifyTouch = false;
        this._tapGestureHandler = null;
        this._doubletapGestureHandler = null;
        this._longpressGestureHandler = null;
        this._pinchGestureHandler = null;
        this._panGestureHandler = null;
        this._rotationGestureHandler = null;
        this._flingGestureHandler = null;
        this._eventData = null;
    }
    onGestureStateChange(type) {
        return (event) => {
            if (this.callback) {
                this.callback.call(this._context, Object.assign(Object.assign({ eventName: gestures_1.toString(type), object: event.data.view, type }, event.data), event.data.extraData));
            }
        };
    }
    onGestureTouchChange(type) {
        return (event) => {
            if (this.callback && event.data.state === gesturehandler_common_1.GestureState.ACTIVE) {
                this.callback.call(this._context, Object.assign(Object.assign({ eventName: gestures_1.toString(type), object: event.data.view, type }, event.data), event.data.extraData));
            }
        };
    }
    _attach(target, type) {
        if (type & gestures_1.GestureTypes.touch) {
            this._notifyTouch = true;
        }
        const manager = gesturehandler_1.Manager.getInstance();
        if (type & gestures_1.GestureTypes.tap) {
            const gestureHandler = (this._tapGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.TAP, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.tap), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.tap), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.longPress) {
            const gestureHandler = (this._longpressGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.LONG_PRESS, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.longPress), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.longPress), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.doubleTap) {
            const gestureHandler = (this._doubletapGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.TAP, TAG++, {
                numberOfTaps: 2,
            }));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.doubleTap), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.doubleTap), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.pinch) {
            const gestureHandler = (this._pinchGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.PINCH, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.pinch), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.pinch), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.swipe) {
            const gestureHandler = (this._flingGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.FLING, TAG++));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.swipe), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.swipe), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.pan) {
            const gestureHandler = (this._panGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.PAN, TAG++, {}));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.pan), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.pan), this);
            gestureHandler.attachToView(target);
        }
        if (type & gestures_1.GestureTypes.rotation) {
            const gestureHandler = (this._rotationGestureHandler = manager.createGestureHandler(gesturehandler_common_1.HandlerType.ROTATION, TAG++, {}));
            gestureHandler.on(gesturehandler_common_1.GestureHandlerStateEvent, this.onGestureStateChange(gestures_1.GestureTypes.rotation), this);
            gestureHandler.on(gesturehandler_common_1.GestureHandlerTouchEvent, this.onGestureTouchChange(gestures_1.GestureTypes.rotation), this);
            gestureHandler.attachToView(target);
        }
    }
    androidOnTouchEvent(motionEvent) {
        if (this._notifyTouch) {
            if (!this._eventData) {
                this._eventData = new TouchGestureEventData();
            }
            this._eventData.prepare(this.target, motionEvent);
            _executeCallback(this, this._eventData);
        }
    }
}
exports.GesturesObserver = GesturesObserver;
class Pointer {
    constructor(id, event) {
        this.event = event;
        this.ios = undefined;
        this.android = id;
    }
    getX() {
        return this.event.getX(this.android) / utils_1.layout.getDisplayDensity();
    }
    getY() {
        return this.event.getY(this.android) / utils_1.layout.getDisplayDensity();
    }
}
class TouchGestureEventData {
    constructor() {
        this.eventName = gestures_1.toString(gestures_1.GestureTypes.touch);
        this.type = gestures_1.GestureTypes.touch;
        this.ios = undefined;
    }
    prepare(view, e) {
        this.view = view;
        this.object = view;
        this.android = e;
        this.action = this.getActionType(e);
        this._activePointers = undefined;
        this._allPointers = undefined;
    }
    getPointerCount() {
        return this.android.getPointerCount();
    }
    getActivePointers() {
        if (!this._activePointers) {
            this._activePointers = [new Pointer(this.android.getActionIndex(), this.android)];
        }
        return this._activePointers;
    }
    getAllPointers() {
        if (!this._allPointers) {
            this._allPointers = [];
            for (let i = 0; i < this.getPointerCount(); i++) {
                this._allPointers.push(new Pointer(i, this.android));
            }
        }
        return this._allPointers;
    }
    getX() {
        return this.getActivePointers()[0].getX();
    }
    getY() {
        return this.getActivePointers()[0].getY();
    }
    getActionType(e) {
        switch (e.getActionMasked()) {
            case android.view.MotionEvent.ACTION_DOWN:
            case android.view.MotionEvent.ACTION_POINTER_DOWN:
                return gestures_1.TouchAction.down;
            case android.view.MotionEvent.ACTION_MOVE:
                return gestures_1.TouchAction.move;
            case android.view.MotionEvent.ACTION_UP:
            case android.view.MotionEvent.ACTION_POINTER_UP:
                return gestures_1.TouchAction.up;
            case android.view.MotionEvent.ACTION_CANCEL:
                return gestures_1.TouchAction.cancel;
        }
        return '';
    }
}
function _executeCallback(observer, args) {
    if (observer && observer.callback) {
        observer.callback.call(observer._context, args);
    }
}
//# sourceMappingURL=gestures_override.mjs.map